'********************************PDQDRIVE.BAS********************************
'JRD NOTE:
'Get one or all valid drives with a FUNCTION for one and a SUB for all.
'12/4/94
'=========================================================================
'
' A Basic program can call interrupts to find out which logical drives
' are available. The interrupts used are 21Hex, function 0EHex (Select
' disk) and 21Hex, function 19Hex (Get current disk). The method
' involves circulating through each possible logical drive (A-Z) and
' calling the interrupt to select that drive. If this is successful,
' subsequently calling the interrupt to get the current drive should
' return that same drive. If it isn't the same, the drive wasn't
' successfully selected and therefore isn't available.
'
' Function 0EH of interrupt 21H (Select disk) requires input from the
' following registers:
'
'    AH (upper byte of AX) = 0EH
'    DL (lower byte of DX) = drive code (0=A, 1=B, etc.)
'
' When called, it makes the drive passed in DL current, if possible. If
' that drive is not available, the interrupt does not return an error.
' That is why function 19H of interrupt 21H (Get current disk) must be
' called to see if function 0EH was successful. Function 19H requires
' the following registers for input:
'
'    AH (upper byte of AX) = 19H
'
' It returns the following:
'
'    AL (lower byte of AX) = current drive code (0=A, 1=B, etc.)
'
' For more information on interrupt 21H, functions 0EH and 19H, see
' Pages 355 and 367 (respectively) of "Advanced MS-DOS Programming,
' Second Edition," by Ray Duncan (Microsoft Press, 1988).
'
' The following program displays all the available logical drives:
'
' 'Remember to invoke QB.EXE (or QBX.EXE) with the /L switch) because
' 'support for CALL Interrupt is in QB.QLB (or QBX.QLB).
'=========================================================================
DEFINT A-Z
CONST True = -1, False = 0
'QuickBASIC can use 10 of the 14 Registers (Built-in integer variables)
TYPE RegType
	 ax        AS INTEGER      'Accumulator       Register
	 bx        AS INTEGER      'Base                 "
	 cx        AS INTEGER      'Count                "
	 dx        AS INTEGER      'Data                 "
	 bp        AS INTEGER      'Base Pointer         "
	 si        AS INTEGER      'Source Index         "
	 di        AS INTEGER      'Destination Index    "
	 flags     AS INTEGER      'Flags                "
	 ds        AS INTEGER      'Data Segment         " (CALL InterruptX)
	 es        AS INTEGER      'Extra Segment        " (CALL InterruptX)
END TYPE

'My Declares in this module
DECLARE SUB CheckAllDrives (ADrive$, IsValidDrive%)
DECLARE SUB Cursor (IfTrue%)
DECLARE FUNCTION ValidDrive% (ADrive$)
DECLARE SUB PressAnyKey ()
DECLARE SUB IsCursor (IfTrue%)
DECLARE SUB DefaultCursor ()
DECLARE SUB LocateIt (Row%, text$)
DECLARE SUB Splash2 (Choice%)
DECLARE SUB TextBoxShadow (Row%, Col%, Message$, Outline%, Shadow%, Length%)
DECLARE FUNCTION Center% (text$)
DECLARE SUB SetBorder (ColrByte%)
DECLARE SUB TwoColrs (Fgd%, Bkg%, Colr%)
DECLARE SUB GetColr (Fgd%, Bkg%, Colr%)
DECLARE SUB DateTime (Row%, Col%, Colr%)
DECLARE FUNCTION OneColr% (Fgd%, Bkg%)

'PDQ Declares
DECLARE SUB INTERRUPT (IntNumber%, Registers AS ANY)
DECLARE SUB INTERRUPTX (IntNumber%, Registers AS ANY)
DECLARE FUNCTION BIOSInput (Work$, EditColr%)
DECLARE SUB PDQPrint (Work$, Row%, Column%, Colr%)
DECLARE SUB PDQCPrint2 (Work$, Row%, Column%)

'QuickPak
DECLARE SUB APrint0 (BYVAL Address%, NumEls%, MidFirst%, NumChars%, Colr%)
DECLARE SUB QPrint0 (Work$, Colr%)
DECLARE SUB Browser (Dr$())
DECLARE FUNCTION OneColor% (Fgd%, Bkgd%)

'Executable code follows
'=======================

REDIM SHARED Box$(1 TO 56)
DIM SHARED Regs AS RegType        'shared
DIM SHARED Dr$(0 TO 25)
'saves the screen colors to restore at end
CALL GetColr(Fgd%, Bkgd%, RestoreColr%)
Again:                  

COLOR 15, 1
CLS
BaseRow% = 7

IsCursor (False)
'Changes the Border color
Count% = Count% + 1
IF Count% >= 16 THEN Count% = 1
CALL SetBorder(Count%)

'Changes the background screen character
SplashCount% = SplashCount% + 1
IF SplashCount% >= 11 THEN SplashCount% = 1
CALL Splash2(SplashCount%)
'Tells you what color the border is
LOCATE 1, 1: PRINT LTRIM$(STR$(Count%))

Message$ = "Check and Verify all Local Drives"
COLOR 15, 4
Row% = 2: Col% = 0: Outline% = 5: Shadow% = True: Length% = False
CALL TextBoxShadow(Row%, Col%, Message$, Outline%, Shadow%, Length%)

COLOR 11, 3
Row% = BaseRow%: Col% = 0: Outline% = 1: Shadow% = True: Length% = 2
Message$ = SPACE$(20)
CALL TextBoxShadow(Row%, Col%, Message$, Outline%, Shadow%, Length%)
text$ = "  Drive Letter to test  "
CALL LocateIt(BaseRow%, text$)
text$ = " OR <Esc> to QUIT "
CALL LocateIt(BaseRow% + 4, text$)
ThisRow% = CSRLIN
ThisCol% = POS(0)
COLOR 11, 0
text$ = SPACE$(3)
CALL LocateIt(BaseRow% + 2, text$)
Cursor (True)
LOCATE BaseRow% + 2, 41
DO
	ADrive$ = INKEY$
LOOP UNTIL LEN(ADrive$)
Cursor (False)
PRINT ADrive$
COLOR 15, 1

IF ASC(ADrive$) = 27 THEN
	CALL SetBorder(4)
	COLOR 13, 4
	Row% = BaseRow% + 7: Col% = 0: Outline% = 5: Shadow% = True: Length% = 2
	text$ = "John De Palma on CompuServe 76076,571"
	Message$ = SPACE$(LEN(text$))
	CALL TextBoxShadow(Row%, Col%, Message$, Outline%, Shadow%, Length%)
	COLOR 15, 4
	CALL LocateIt(BaseRow% + 9, text$)
   
	CALL TwoColrs(Fgd%, Bkg%, RestoreColr%)
	PressAnyKey
	DefaultCursor
	CALL SetBorder(0)
	COLOR Fgd%, Bkg%
	CLS
	END
END IF

'in case you press {Enter} and pass a null string
IF ASC(ADrive$) <= 64 THEN
	BEEP
	GOTO Again
END IF

'This is a function that returns True if it is a valid drive
PrintAdrive$ = UCASE$(ADrive$) + ":"
'LOCATE BaseRow% + 4, 20
IF ValidDrive%(ADrive$) THEN
	Message$ = "The Drive " + PrintAdrive$ + " is VALID"
ELSE
	Message$ = "The Drive " + PrintAdrive$ + " is =NOT= VALID"
END IF
Row% = BaseRow% + 7: Col% = 0: Outline% = 2: Shadow% = True: Length% = False
CALL TextBoxShadow(Row%, Col%, Message$, Outline%, Shadow%, Length%)

CALL PressAnyKey

'This is a SUB that stores all the valid drives in an array,
'will be useful if and when I do a Slack space program.
CALL CheckAllDrives(ADrive$, IsValidDrive%)
COLOR 14, 6
Row% = BaseRow% + 11: Col% = 0: Outline% = 3: Shadow% = True: Length% = 3
DummyMessage$ = SPACE$(LEN(SPACE$(23)))
CALL TextBoxShadow(Row%, Col%, DummyMessage$, Outline%, Shadow%, Length%)
COLOR 15, 6
text$ = "<Esc> Key to EXIT Scroll"
CALL LocateIt(BaseRow% + 11 + Length% + 2, text$)
DateColr% = OneColor%(10, 3)
CALL DateTime(12, 2, DateColr%)
CALL Browser(Dr$())
IsCursor (False)

'CALL PressAnyKey
GOTO Again

SUB Browser (Dr$())
'for PDQDRIVE.BAS; uses zero as the first Array element
FirstEl% = 0
FirstChar% = 1
NumDrives% = 26
NumEl% = 3
NumChar% = 23
BaseRow% = 20
BaseCol% = 41 - (NumChar% / 2)
ArrayColor% = OneColor%(14, 6)

HelpColor% = OneColor%(14, 2)
DO
	LOCATE BaseRow%, BaseCol%
	'This next is an assembly SUB from Crescent's QuickPAK
	CALL APrint0(BYVAL VARPTR(Dr$(FirstEl%)), NumEl%, FirstChar%, NumChar%, ArrayColor%)
	Row$ = STR$(FirstEl%)
	Help$ = "[" + LTRIM$(Row$) + "]   Home End    "
	LOCATE BaseRow% - 1, BaseCol%
	CALL QPrint0(Help$, HelpColor%)
	DO
		X$ = INKEY$
		IF X$ = CHR$(27) THEN EXIT SUB
	LOOP UNTIL LEN(X$) = 2

	SELECT CASE ASC(RIGHT$(X$, 1))
	  
		CASE 80, 75                    'down or left arrow
			IF FirstEl% < NumDrives% - NumEl% THEN
				FirstEl% = FirstEl% + 1
			ELSEIF FirstEl% >= NumDrives% - NumEl% THEN
				FirstEl% = NumDrives% - NumEl%
				PLAY "p32"              'a click
			ELSE
			END IF
	  
		CASE 72, 77                    'up or right arrow
			IF FirstEl% > 0 THEN FirstEl% = FirstEl% - 1

	
		CASE 71                    'Home
			IF FirstEl% > 1 THEN FirstEl% = 0
			IF FirstChar > 1 THEN FirstChar = 0
	   
		CASE 79                     'End
			FirstEl% = NumDrives% - NumEl%

		CASE ELSE
			 BEEP
	END SELECT

LOOP

END SUB

FUNCTION Center% (text$)
	Center% = 41 - LEN(text$) \ 2
END FUNCTION

SUB CheckAllDrives (ADrive$, IsValidDrive%)
ADrive$ = UCASE$(ADrive$)
'DIM Dr$(25)
FOR Drive% = 0 TO 25                  'Circulate through drives A-Z.

   Regs.ax = &HE00                    'Function 0EH (AH = 0EH).
   Regs.dx = Drive%
   CALL INTERRUPT(&H21, Regs)   'Select a drive, if it's there.

   Regs.ax = &H1900                   'Function 19H (AH = 19H).
   CALL INTERRUPT(&H21, Regs)   'Get currently selected drive.
   Sp$ = SPACE$(1)
   'JRD NOTE: my stuff next
   IF (Regs.ax AND &HFF) = Drive% THEN    '(Regs.ax AND &HFF) = AL.
	  Dr$(Drive%) = Sp$ + CHR$(Drive% + 65) + ": is VALID"
   ELSE
	  Dr$(Drive%) = Sp$ + CHR$(Drive% + 65) + ": NOT a Valid Drive"
   END IF
   'STOP
   FOR i% = 0 TO 25
		IF Dr$(i) = ADrive$ THEN
			IsValidDrive% = True
			EXIT SUB
		END IF
	NEXT
NEXT Drive%
			IsValidDrive% = False
END SUB

SUB Cursor (IfTrue%)
	'Like this better than two SUBs of CursorOn and CursorOff
	'Especially with the trouble changing the code with QuickPak
	'and PDQ
	IF IfTrue% = True THEN
		LOCATE , , 1, 4, 7
	ELSE
		LOCATE , , 0
	END IF
END SUB

SUB DateTime (Row%, Col%, Colr%)
   
	CALL TwoColrs(Fgd%, Bkg%, Colr%)
	COLOR Fgd%, Bkg%
	text$ = "DATE: " + DATE$ + " Time: " + TIME$
	Message$ = SPACE$(LEN(text$) \ 2)
	'TextBoxShadow (Row%, Col%, Message$, Outline%, Shadow%, Length%)
	CALL TextBoxShadow(Row%, Col%, Message$, 0, False, 1)
	LOCATE Row% + 1, Col% + 1: PRINT "DATE: "; DATE$
	LOCATE Row% + 2, Col% + 1: PRINT "Time: "; TIME$

END SUB

SUB DefaultCursor
	LOCATE , , 1, 6, 7
END SUB

SUB GetColr (Fgd%, Bkg%, Colr%) STATIC

	Colr% = SCREEN(1, 1, 1)

	Fgd% = (Colr% AND 128) \ 8 + (Colr% AND 15)
	Bkg% = (Colr% AND 112) \ 16

END SUB

SUB IsCursor (IfTrue%)
	'Like this better than two SUBs of CursorOn and CursorOff
	'Especially with the trouble changing the code with QuickPak
	'and PDQ
	IF IfTrue% = True THEN
		'LOCATE , , 1, 4, 7     'big cursor
		LOCATE , , 1
	ELSE
		LOCATE , , 0
	END IF
END SUB

SUB LocateIt (Row%, text$)
	 LOCATE Row%, 41 - (LEN(text$)) \ 2
	 PRINT text$;
END SUB

FUNCTION OneColr% (Fgd%, Bkg%)
	OneColr% = (Fgd% AND 16) * 8 + ((Bkg% AND 7) * 16 + (Fgd% AND 15))
END FUNCTION

SUB PDQCPrint2 (Work$, Row, Column) STATIC

	SaveRow = CSRLIN
	SaveCol = POS(0)
	LOCATE Row, Column
	PRINT Work$;
	LOCATE SaveRow, SaveCol

END SUB

SUB PressAnyKey
   
	text$ = "  PRESS: A Key to Continue...  "
	Row% = CSRLIN
	Col% = POS(0)
	Colr% = SCREEN(Row%, Col%, 1)
	CALL TwoColrs(Fgd%, Bkgd%, Colr%)
	COLOR 11, 0
	LOCATE 24, Center(text$)
	GOSUB StoreText
   
	PRINT text$;
	COLOR Fgd%, Bkgd%
	WHILE INKEY$ = "": WEND
	WHILE INKEY$ <> "": WEND        'need this to remove the key press
	LOCATE 24, Center(text$)
	PRINT StoreScreenLine$;
	'PRINT SPACE$(LEN(text$));
	LOCATE Row%, Col%
	EXIT SUB
StoreText:
   
	FOR i% = 1 TO LEN(text$)
		StoreScreenLine$ = StoreScreenLine$ + CHR$(SCREEN(CSRLIN, i%))
	NEXT
	RETURN
END SUB

SUB SetBorder (ColrByte%) STATIC

	'DIM Regs AS RegType
	Regs.ax = &H1001
	Regs.bx = ColrByte% * &H100
	CALL INTERRUPT(&H10, Regs)

END SUB

SUB Splash2 (Choice%)
'prints a full screen of: 
'                         

SELECT CASE Choice%
	CASE 1
		Char1% = 244
		Char2% = 245
	CASE 2
		Char1% = 174
		Char2% = 175
	CASE 3
		Char1% = 242
		Char2% = 243
	CASE 4
		Char1% = 47
		Char2% = 92
	CASE 5
		Char1% = 220
		Char2% = 240
	CASE 6
		Char1% = 221
		Char2% = 222
	CASE 7
		Char1% = 180
		Char2% = 195
	CASE 8
		Char1% = 254
		Char2% = 222
	CASE 9
		Char1% = 146
		Char2% = 158
	CASE 10
		Char1% = 159
		Char2% = 159
	CASE ELSE
		Char1% = 244
		Char2% = 245
END SELECT

	FOR i = 1 TO 25 STEP 2
		LOCATE i, 1
		PRINT STRING$(80, Char1%);
	NEXT
	FOR i = 2 TO 24 STEP 2
		LOCATE i, 1
		PRINT STRING$(80, Char2%);
	NEXT

END SUB

SUB TextBoxShadow (Row%, Col%, Message$, Outline%, Shadow%, Length%)
  
	'Adding other ancillary SUBs as "GOSUB"s so that they
	'do not need to be imported into a new module.
	'edited to omit the SUB TwoColr(Fgd%,Bkg,Colr%)
	'and tested Sun  08-21-1994  21:25:34

	'got to have a REDIM SHARED Box$(1 to 56) in main module
	'Other SUB/FUNCTIONs still needed for this SUB are:
	'SUB
	'LocateIt(Row%, Text$)
	'FUNCTION
	'Center% (text$)
	'
	'Puts a message into a three line box -or-
	'draw a box without a message using Message$=SPACE$(x)
	'where "x" is the width of the box and Length%= number of lines > 3
	'Boxes are centered if Col% = 0; else left side of box = Col%.
	'Boxes display a true shadow if Shadow% <> 0
	'True = -1: False = 0

	STATIC BoxReadFlag
	Message$ = LEFT$(Message$, 60)
	BoxWidth% = LEN(Message$) + 4
	SELECT CASE Outline%
		CASE 0
			j = 8 * 6 + 1
		CASE 1
			j = 1
		CASE 2
			j = 8 + 1
		CASE 3
			j = 8 * 2 + 1
		CASE 4
			j = 8 * 3 + 1
		CASE 5
			j = 8 * 4 + 1
		CASE 6
			j = 8 * 5 + 1
		CASE ELSE
			j = 8 * 6 + 1
	END SELECT

	IF BoxReadFlag THEN GOTO Skip
	REDIM Box$(1 TO 56)
	BoxReadFlag = True

'single line box
	Box$(1) = ""
	Box$(2) = ""
	Box$(3) = ""
	Box$(4) = ""
	Box$(5) = ""
	Box$(6) = ""
	Box$(7) = ""
	Box$(8) = ""

'double top box
	Box$(9) = ""
	Box$(10) = ""
	Box$(11) = ""
	Box$(12) = ""
	Box$(13) = ""
	Box$(14) = ""
	Box$(15) = ""
	Box$(16) = ""

'double side box
	Box$(17) = ""
	Box$(18) = ""
	Box$(19) = ""
	Box$(20) = ""
	Box$(21) = ""
	Box$(22) = ""
	Box$(23) = ""
	Box$(24) = ""

'double box
	Box$(25) = ""
	Box$(26) = ""
	Box$(27) = ""
	Box$(28) = ""
	Box$(29) = ""
	Box$(30) = ""
	Box$(31) = ""
	Box$(32) = ""

'bold box
	Box$(33) = ""
	Box$(34) = ""
	Box$(35) = ""
	Box$(36) = ""
	Box$(37) = ""
	Box$(38) = ""
	Box$(39) = ""
	Box$(40) = ""

'bold and thick box
	Box$(41) = ""
	Box$(42) = ""
	Box$(43) = ""
	Box$(44) = ""
	Box$(45) = ""
	Box$(46) = ""
	Box$(47) = ""
	Box$(48) = ""

'no box
	Box$(49) = " "
	Box$(50) = " "
	Box$(51) = " "
	Box$(52) = " "
	Box$(53) = " "
	Box$(54) = " "
	Box$(55) = " "
	Box$(56) = " "

Skip:

   
  
	IF Col% = 0 THEN

			BoxText$ = Box$(j) + STRING$(BoxWidth%, Box$(j + 1)) + Box$(j + 2)
			GOSUB Location
			'CALL LocateIt(Row%, BoxText$)
			Row2% = CSRLIN: Col2% = POS(0)
			Colr% = SCREEN(Row2%, Col2% - 1, 1)
			'GOSUB GetTwoColors
			CALL TwoColrs(Fgd%, Bkg%, Colr%)
   
			FOR i = 1 TO Length% + 1
			BoxText$ = Box$(j + 3) + "  " + Message$ + "  " + Box$(j + 4)
			'GOSUB Location
			CALL LocateIt(Row% + i, BoxText$)

			IF Shadow% THEN
				COLOR 7, 0
					FOR k = 1 TO 2
						PRINT CHR$(SCREEN(CSRLIN, POS(0)));
					NEXT
				COLOR Fgd%, Bkg%
			END IF
			NEXT i

			BoxText$ = Box$(j + 5) + STRING$(BoxWidth%, Box$(j + 6)) + Box$(j + 7)
			'GOSUB Location
			CALL LocateIt(Row% + i, BoxText$)
	 
			IF Shadow% THEN
				COLOR 7, 0
				FOR k = 1 TO 2
					PRINT CHR$(SCREEN(CSRLIN, POS(0)));
				NEXT
			'COLOR Fgd%, Bkg%
   
			COLOR 7, 0
			LOCATE Row% + i + 1, Center(BoxText$) + 2
	 
				FOR k = 1 TO BoxWidth% + 2
					PRINT CHR$(SCREEN(CSRLIN, POS(0)));
				NEXT
				COLOR Fgd%, Bkg%
			END IF
	ELSE

			BoxText$ = Box$(j) + STRING$(BoxWidth%, Box$(j + 1)) + Box$(j + 2)
			LOCATE Row%, Col%
			PRINT BoxText$;
			Row2% = CSRLIN: Col2% = POS(0)
			Colr% = SCREEN(Row2%, Col2% - 1, 1)
			GOSUB GetTwoColors
						'CALL TwoColrs(Fgd%, Bkg%, Colr%)

			FOR i = 1 TO Length% + 1
			BoxText$ = Box$(j + 3) + "  " + Message$ + "  " + Box$(j + 4)
			LOCATE Row% + i, Col%
			PRINT BoxText$;
	 
			IF Shadow% THEN
				COLOR 7, 0
				FOR k = 1 TO 2
					PRINT CHR$(SCREEN(CSRLIN, POS(0)));
				NEXT
				COLOR Fgd%, Bkg%
			END IF
	 
			NEXT i

			BoxText$ = Box$(j + 5) + STRING$(BoxWidth%, Box$(j + 6)) + Box$(j + 7)
			LOCATE Row% + i, Col%
			PRINT BoxText$;
	 
			IF Shadow% THEN
				COLOR 7, 0
				FOR k = 1 TO 2
					PRINT CHR$(SCREEN(CSRLIN, POS(0)));
				NEXT
			  
				LOCATE Row% + i + 1, Col% + 2
				FOR k = 1 TO BoxWidth% + 2
					PRINT CHR$(SCREEN(CSRLIN, POS(0)));
				NEXT
				COLOR Fgd%, Bkg%
			END IF

	END IF
	EXIT SUB
GetTwoColors:
	Fgd% = (Colr% AND 128) \ 8 + (Colr% AND 15)
	Bkg% = (Colr% AND 112) \ 16
RETURN

Location:
	LOCATE Row%, 41 - (LEN(BoxText$)) \ 2
	PRINT BoxText$;
RETURN

END SUB

SUB TwoColrs (Fgd%, Bkg%, Colr%)

	Fgd% = (Colr% AND 128) \ 8 + (Colr% AND 15)
	Bkg% = (Colr% AND 112) \ 16

END SUB

FUNCTION ValidDrive% (ADrive$)

IF ADrive$ = "" THEN ADrive$ = "C"
Drive% = ASC(UCASE$(ADrive$)) - 65  'convert drive letter to number
									'A = 0, B = 1, C = 2, etc...
   Regs.ax = &HE00                    'Function 0EH (AH = 0EH).
   Regs.dx = Drive%
   CALL INTERRUPT(&H21, Regs)   'Select a drive, if it's there.

   Regs.ax = &H1900                   'Function 19H (AH = 19H).
   CALL INTERRUPT(&H21, Regs)   'Get currently selected drive.

   'JRD NOTE: my stuff next
   IF (Regs.ax AND &HFF) = Drive% THEN    '(Regs.ax AND &HFF) = AL.
			ValidDrive% = True
	  ELSE
			ValidDrive% = False
	  END IF

END FUNCTION

